unit unit1;

{$mode objfpc}{$H+}

interface

uses
  LCLIntf, Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, LCLType,
  StdCtrls, ComCtrls, LMessages, LCLProc;

type

  { TDummyControl }

  TDummyControl = class (TCustomControl)
  private
    FSkipPaint: Boolean;
    FUpdateRect: TRect;
    FUseTempBitmap: Boolean;
    FClearing: Boolean;
    procedure SetSkipPaint(const AValue: Boolean);
    procedure SetUseTempBitmap(const AValue: Boolean);
  protected
    procedure WMPaint(var Message: TLMPaint); message LM_PAINT;
    procedure WMEraseBkgnd(var Message: TLMEraseBkgnd); message LM_ERASEBKGND;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Clear;
    property SkipPaint: Boolean read FSkipPaint write SetSkipPaint;
    property UseTempBitmap: Boolean read FUseTempBitmap write SetUseTempBitmap;
    property UpdateRect: TRect read FUpdateRect write FUpdateRect;
  end;
  
  { TForm1 }

  TForm1 = class(TForm)
    ButtonShowClientToScreen: TButton;
    ButtonShowUpdateRect: TButton;
    ButtonShowClientBounds: TButton;
    ButtonClear: TButton;
    ButtonShowClientRect: TButton;
    ButtonInvalidateRect: TButton;
    CheckBorderStyle: TCheckBox;
    CheckSkipPaint: TCheckBox;
    CheckBufferedPaint: TCheckBox;
    StatusBarMain: TStatusBar;
    procedure ButtonClearClick(Sender: TObject);
    procedure ButtonInvalidateRectClick(Sender: TObject);
    procedure ButtonShowClientBoundsClick(Sender: TObject);
    procedure ButtonShowClientRectClick(Sender: TObject);
    procedure ButtonShowClientToScreenClick(Sender: TObject);
    procedure ButtonShowUpdateRectClick(Sender: TObject);
    procedure CheckBorderStyleChange(Sender: TObject);
    procedure CheckBufferedPaintChange(Sender: TObject);
    procedure CheckSkipPaintChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { private declarations }
    FDummy: TDummyControl;
  public
    { public declarations }
  end; 

var
  Form1: TForm1; 

implementation

{ TDummyControl }

procedure TDummyControl.Clear;
begin
  FClearing := True;
  Invalidate;
end;

constructor TDummyControl.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

procedure TDummyControl.Paint;

  procedure PaintToCanvas(ACanvas: TCanvas);
  begin
    with ACanvas do
    begin
      Pen.Width := 2;
      Pen.Color := clRed;
      MoveTo(0, 1);
      LineTo(Width, 1);
      Pen.Color := clYellow;
      MoveTo(0, 3);
      LineTo(Width, 3);
      Pen.Color := clBlue;
      MoveTo(0, 5);
      LineTo(Width, 5);
      //paint the remaining area
      Brush.Color := clFuchsia;
      FillRect(0, 6, ClientWidth, ClientHeight);
    end;
  end;

var
  TempBitmap: TBitmap;
begin
  if FSkipPaint then
    Exit;
  Canvas.Brush.Color := clWhite;
  Canvas.FillRect(0, 0, Width, Height);
  if FClearing then
  begin
    FClearing := False;
    Exit;
  end;

  if FUseTempBitmap then
  begin
    TempBitmap := TBitmap.Create;
    TempBitmap.Width := ClientWidth;
    TempBitmap.Height := ClientHeight;
    PaintToCanvas(TempBitmap.Canvas);
    BitBlt(Canvas.Handle, 0, 0, ClientWidth, ClientHeight,
      TempBitmap.Canvas.Handle, 0, 0, SRCCOPY);
    TempBitmap.Destroy;
  end
  else
    PaintToCanvas(Canvas);
end;

procedure TDummyControl.SetSkipPaint(const AValue: Boolean);
begin
  if FSkipPaint=AValue then exit;
  FSkipPaint:=AValue;
end;


procedure TDummyControl.SetUseTempBitmap(const AValue: Boolean);
begin
  if FUseTempBitmap=AValue then exit;
  FUseTempBitmap:=AValue;
end;

procedure TDummyControl.WMEraseBkgnd(var Message: TLMEraseBkgnd);
begin
  Message.Result := 1;
end;

procedure TDummyControl.WMPaint(var Message: TLMPaint);
begin
  DebugLn('Dummy WMPaint');
  with Message.PaintStruct^.rcPaint do
    DebugLn('Message.PaintStruct^.rcPaint: Left: %d Top: %d Right: %d Bottom: %d', [Left, Top, Right, Bottom]);
  FUpdateRect := Message.PaintStruct^.rcPaint;
  inherited WMPaint(Message);
end;

{ TForm1 }

procedure TForm1.ButtonClearClick(Sender: TObject);
begin
  FDummy.Clear;
end;

procedure TForm1.ButtonInvalidateRectClick(Sender: TObject);
var
  R: TRect;
begin
  R := Rect(0, 0, FDummy.ClientWidth, 6);
  InvalidateRect(FDummy.Handle, @R, True);
end;

procedure TForm1.ButtonShowClientBoundsClick(Sender: TObject);
var
  R: TRect;
begin
  GetClientBounds(FDummy.Handle, R);
  StatusBarMain.SimpleText := Format('ClientBounds: Left: %d Top: %d Right: %d Bottom: %d', [R.Left, R.Top, R.Right, R.Bottom]);
end;

procedure TForm1.ButtonShowClientRectClick(Sender: TObject);
var
  R: TRect;
begin
  LCLIntf.GetClientRect(FDummy.Handle, R);
  StatusBarMain.SimpleText := Format('ClientRect: Left: %d Top: %d Right: %d Bottom: %d', [R.Left, R.Top, R.Right, R.Bottom]);
end;

procedure TForm1.ButtonShowClientToScreenClick(Sender: TObject);
var
  P, P2: TPoint;
begin
  P := Point(0,0);
  lclintf.ClienttoScreen(FDummy.Handle, P);
  P2 := Point(0,0);
  lclintf.ClienttoScreen(Form1.Handle, P2);
  StatusBarMain.SimpleText := Format('ClientToScreen(0,0): FDummy x: %d y:%d - Form1 x: %d y:%d', [P.x, P.y, P2.x, P2.y]);
end;

procedure TForm1.ButtonShowUpdateRectClick(Sender: TObject);
begin
  with FDummy.UpdateRect do
    StatusBarMain.SimpleText := Format('UpdateRect: Left: %d Top: %d Right: %d Bottom: %d', [Left, Top, Right, Bottom]);
end;

procedure TForm1.CheckBorderStyleChange(Sender: TObject);
const
  BorderStyleMap: array[Boolean] of TBorderStyle = (bsNone, bsSingle);
begin
  FDummy.BorderStyle := BorderStyleMap[CheckBorderStyle.Checked];
end;

procedure TForm1.CheckBufferedPaintChange(Sender: TObject);
begin
  FDummy.UseTempBitmap := CheckBufferedPaint.Checked;
  FDummy.Invalidate;
end;

procedure TForm1.CheckSkipPaintChange(Sender: TObject);
begin
  FDummy.SkipPaint := CheckSkipPaint.Checked;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FDummy := TDummyControl.Create(Self);
  with FDummy do
  begin
    Name := 'Dummy';
    Parent := Self;
    SetBounds(0, 0, 100, 100);
  end;
end;

initialization
  {$I unit1.lrs}

end.

